# Approximate Inversion of generalized Radon Transforms - Software package
This software package contains the implementation of the approximate inverse for 2D linear seismic imaging using the common offset acquisition geometry, written in Python and C++. It was only tested in a Linux environment.


## Requirements
On all platforms, we recommend a preassembled scientific python distribution, such as [Continuum IO’s Anaconda](https://www.anaconda.com/products/distribution).

- Python 3.7 (or greater), we are currently using Python 3.9
- C++11 compiler (e.g. MinGW for Windows, g++ for Linux, XCode for MacOS)
- NumPy 1.7 (or greater), smaller than 1.24 (otherwise pysit will currently not work), see https://numpy.org/
- SciPy, see https://scipy.org/
- matplotlib 1.3 (or greater), see https://matplotlib.org/
- eikonalfm, see https://github.com/kevinganster/eikonalfm
- scikit-image, see https://scikit-image.org/docs/stable/install.html

### Additional requirements for data from a wave solver (optional)
- PySIT (we recommend following this manual: https://pysit.readthedocs.io/en/latest/install.html)
    - PyAMG 2.05 (or greater)
    - ObsPy 0.85 (or greater)
- MPI4Py, see https://mpi4py.readthedocs.io/en/stable/install.html

---

To use this package, we recomend using a [virtual environment](https://docs.python.org/3/library/venv.html). To setup the python environment with the neccessary modules, run:

```bash
python -m venv env
source env/bin/activate

pip install numpy==1.23
pip install eikonalfm
pip install scikit-image
pip install matplotlib
```

for inconsistent reconstructions (see below), you will also need the following:

```bash
pip install mpi4py
pip install git+https://github.com/pysit/pysit.git
```

Alternatively for a new [Anaconda environment](https://docs.conda.io/projects/conda/en/latest/user-guide/tasks/manage-environments.html):

```bash
conda create -n approxinv numpy=1.23 matplotlib scikit-image mpi4py
conda activate approxinv

pip install eikonalfm
pip install git+https://github.com/pysit/pysit.git
```

---

Before your first usage of this package, you will have to compile the C++ extension `amplitude`:

```bash
cd lib_amplitude
python setup_inplace.py
cd ..
```

This will compile the `amplitude`-module locally in the `lib_amplitude/amplitude/` folder. To install it in the current Python-environment you can instead run

```bash
cd lib_amplitude
pip install .
cd ..
```


## Running reconstructions
### Consistent data
Simply run the script `consistent_reconstruct.py` with a configfile as parameter. You can find different examples in the folder `configs/consistent/`. Optionally an integer can be supplied as second argument to reuse previously calculated reference kernels for a new reconstruction (see below), presuming the config fits the one used to calculate those kernels.

For example:

```bash
python consistent_reconstruct.py configs/consistent/linear.ini
```

The resulting images will be saved into a folder `figures/`. The output at termination of the program will be something like

```bash
Image saved to ".../figures/linear/rec1.pdf"
```

indicating where those images can be found.

In the console-output you will also find a line similar to the following:

```bash
saving data to .../saved/linear/kernels1.pcl
```

The number in the filename (in this case `1`) can be used to reuse those reference kernels for a new reconstruction. After adjusting some entries in `configs/consistent/linear.ini` that don't change the reference kernels, we can execute the following:

```bash
python consistent_reconstruct.py configs/consistent/linear.ini 1
```

The reference kernels will simply be loaded from the corresponding file generated earlier, saving a lot of computation time.


### Data from a wave solver
For inconsistent data, we have to do the reconstruction in two steps. First, we have to generate the data by using a wave solver, in our case the module PySIT.
The script uses MPI to utilize multiple processor cores, we therefore have to call it slightly different than earlier:

```bash
mpiexec python inconsistent_datagen.py configs/inconsistent/linear.ini
```

(for more information on how to use MPI4Py, see https://mpi4py.readthedocs.io/en/stable/tutorial.html#running-python-scripts-with-mpi)

This will generate a "pickled" file with the needed data for the reconstruction inside the `saved/` folder. The console output will be something like this:

```bash
saving data to .../saved/inconsistent/g1.pcl
```

where we have to keep in mind the number after `g`, in the above case `1` (this is needed to tell the reconstruction program which file to load the data from).
Next we can reconstruct using the generated data:

```bash
python inconsistent_reconstruct.py configs/inconsistent/linear.ini 1
```

Make sure to use the same configfile and the number from the output of `inconsistent_datagen.py`.  
Again, the output at termination gives the location of the created images:

```bash
Image saved to .../figures/inconsistent/rec1_1.pdf
```
