#pragma once

#include "sweeping.hpp"


/**
 * solves the (factored) transport equation 2 nabla(tau0 * tau1) * nabla(a0 * a1) + (a0 * a1) * lapl(tau0 * tau1) = 0 for a1,
 * where tau0 = |x - x_s| and a0 = 1 / sqrt(|x - x_s|)
 */
template <int NDIM>
class FactoredAmplitude: public Hamiltonian<NDIM>
{
private:
    const double *const c, *const tau_1;
    double *const Hdu;
    std::array<double *, NDIM> Hdp;

public:
    FactoredAmplitude(const SweepInfo<NDIM> &info, const double* const c, const double* const tau_1):
        c(c),
        tau_1(tau_1),
        Hdu(new double[info.size])
    {
        for (int d=0; d < NDIM; d++)
        {
            Hdp[d] = new double[info.size];
        }
    }

    ~FactoredAmplitude()
    {
        delete[] Hdu;
        for (int d=0; d < NDIM; d++)
        {
            delete[] Hdp[d];
        }
    }

private:
    void initialize(const SweepInfo<NDIM> &info, const usize x0, double *const res, std::array<std::vector<double>, NDIM> &alpha) override;
    // the actual hamiltonian
    auto H(const usize x, const double u, const double (&du)[NDIM]) const -> double override;
};


// template <int NDIM>
// class FactoredEikonal: public Hamiltonian<NDIM>
// {
// private:
//     const double *const c;
//     std::array<double*, NDIM> X;

// public:
//     FactoredEikonal(const SweepInfo<NDIM> &info, const double* const c):
//         c(c)
//     {
//         for (int d=0; d < NDIM; d++)
//         {
//             X[d] = new double[info.size];
//         }
//     }

//     ~FactoredEikonal()
//     {
//         for (int d=0; d < NDIM; d++)
//         {
//             delete[] X[d];
//         }
//     }

// private:
//     void initialize(const SweepInfo<NDIM> &info, const usize x0, double *const res, std::array<std::vector<double>, NDIM> &alpha) override;
//     // the actual hamiltonian
//     auto H(const usize x, const double u, const double (&du)[NDIM]) const -> double override;
// };