from typing import Tuple
import numpy as np


# use classes so that everythin is pickle'able
class Laplace:
    def __init__(self, k: int, gamma: float):
        """
        K* = ∆

        Parameters
        ----------
            k : int
                Parameter for C_gamma_k
            gamma : float
                mollifier width
        """
        self.k = k
        self._C_gamma_k = (k + 1) / (np.pi * gamma**(2*k + 2))
        self._gamma_sq = gamma**2

    def __call__(self, x: np.ndarray, p: Tuple[float, float]) -> np.ndarray:
        d_sq = np.sum((x - p)**2, axis=1)

        mask = d_sq < self._gamma_sq
        d_sq_masked = d_sq[mask]

        res = np.zeros_like(d_sq)
        # Grathwohl diss p.112 + applying K-operator
        res[mask] = self._C_gamma_k * 4*self.k * (self._gamma_sq - d_sq_masked)**(self.k-2) * (self.k*d_sq_masked - self._gamma_sq)

        return res

# use classes so that everythin is pickle'able
class x2_Laplace:
    def __init__(self, k: int, gamma: float, power: int=1):
        """
        K* = x_2 ∆

        Parameters
        ----------
            k : int
                Parameter for C_gamma_k
            gamma : float
                mollifier width
            power : int (optional)
                power of x2
        """
        self.k = k
        self.power = power
        self._C_gamma_k = (k + 1) / (np.pi * gamma**(2*k + 2))
        self._gamma_sq = gamma**2

    def __call__(self, x: np.ndarray, p: Tuple[float, float]) -> np.ndarray:
        d_sq = np.sum((x - p)**2, axis=1)

        mask = d_sq < self._gamma_sq
        d_sq_masked = d_sq[mask]
        x2_masked = x[mask, 1]**self.power

        res = np.zeros_like(d_sq)
        # Grathwohl diss p.112 + applying K-operator
        #res[mask] = C_gamma_k * 4*k * (gamma_sq - d_sq_masked)**(k-2) * ((alpha + x2_masked) * (k*d_sq_masked - gamma_sq) + (x2_masked - p[1]) * (d_sq_masked - gamma_sq))
        res[mask] = self._C_gamma_k * 4*self.k * x2_masked * (self._gamma_sq - d_sq_masked)**(self.k-2) * (self.k*d_sq_masked - self._gamma_sq)
        
        return res
